@@ -25,6 +25,7 @@
 #include "tracerinterface.h"
 
 #include <klocale.h>
+
 #include <solid/networking.h>
 
 class QSettings;
@@ -48,13 +49,14 @@ class AgentBasePrivate : public QObject
     void slotWarning( const QString& message );
     void slotError( const QString& message );
     void slotNetworkStatusChange( Solid::Networking::Status );
+    void slotResumedFromSuspend();
 
     virtual void changeProcessed();
 
     QString defaultReadyMessage() const
     {
       if ( mOnline )
-        return i18nc( "@info:status, application ready for work", "Ready" );
+        return i18nc( "@info:status Application ready for work", "Ready" );
       return i18nc( "@info:status", "Offline" );
     }
 
@@ -65,13 +67,20 @@ class AgentBasePrivate : public QObject
 
     QString defaultErrorMessage() const
     {
-      return i18nc( "@info:status", "Error!" );
+      return i18nc( "@info:status", "Error." );
     }
 
+    void setProgramName();
+
     AgentBase *q_ptr;
     Q_DECLARE_PUBLIC( AgentBase )
 
     QString mId;
+    QString mName;
+    QString mResourceTypeName;
+
+    /// Use sessionBus() to access the connection.
+    QDBusConnection mDBusConnection;
 
     int mStatusCode;
     QString mStatusMessage;
@@ -84,7 +93,7 @@ class AgentBasePrivate : public QObject
 
     QSettings *mSettings;
 
-    ChangeRecorder *mMonitor;
+    ChangeRecorder *mChangeRecorder;
 
     org::freedesktop::Akonadi::Tracer *mTracer;
 
@@ -92,16 +101,19 @@ class AgentBasePrivate : public QObject
 
   protected Q_SLOTS:
     virtual void itemAdded( const Akonadi::Item &item, const Akonadi::Collection &collection );
-
     virtual void itemChanged( const Akonadi::Item &item, const QSet<QByteArray> &partIdentifiers );
-
+    virtual void itemMoved( const Akonadi::Item &, const Akonadi::Collection &source, const Akonadi::Collection &destination );
     virtual void itemRemoved( const Akonadi::Item &item );
+    void itemLinked( const Akonadi::Item &item, const Akonadi::Collection &collection );
+    void itemUnlinked( const Akonadi::Item &item, const Akonadi::Collection &collection );
 
     virtual void collectionAdded( const Akonadi::Collection &collection, const Akonadi::Collection &parent );
-
     virtual void collectionChanged( const Akonadi::Collection &collection );
-
+    virtual void collectionChanged( const Akonadi::Collection &collection, const QSet<QByteArray> &partIdentifiers );
+    virtual void collectionMoved( const Akonadi::Collection &collection, const Akonadi::Collection &source, const Akonadi::Collection &destination );
     virtual void collectionRemoved( const Akonadi::Collection &collection );
+    void collectionSubscribed( const Akonadi::Collection &collection, const Akonadi::Collection &parent );
+    void collectionUnsubscribed( const Akonadi::Collection &collection );
 };
 
 }
