@@ -57,10 +57,13 @@ class Session;
  * (3b) resource filters (f) (collections only)
  * (4) item is monitored (+)
  * (5) collection is monitored (+)
- * 
+ *
  * Optionally, the changed objects can be fetched automatically from the server.
  * To enable this, see itemFetchScope() and collectionFetchScope().
  *
+ * Note that as a consequence of rule 3a, it is not possible to monitor (more than zero resources
+ * OR more than zero mimetypes) AND more than zero collections.
+ *
  * @todo Distinguish between monitoring collection properties and collection content.
  * @todo Special case for collection content counts changed
  *
@@ -182,6 +185,33 @@ class AKONADI_EXPORT Monitor : public QObject
      */
     void setItemFetchScope( const ItemFetchScope &fetchScope );
 
+
+    /**
+     * Instructs the monitor to fetch only those parts that were changed and
+     * were requested in the fetch scope.
+     *
+     * This is taken in account only for item modifications.
+     * Example usage:
+     * @code
+     *   monitor->itemFetchScope().fetchFullPayload( true );
+     *   monitor->fetchChangedOnly(true);
+     * @endcode
+     *
+     * In the example if an item was changed, but its payload was not, the full
+     * payload will not be retrieved.
+     * If the item's payload was changed, the monitor retrieves the changed
+     * payload as well.
+     *
+     * The default is to fetch everything requested.
+     *
+     * @since 4.8
+     *
+     * @param enable @c true to enable the feature, @c false means everything
+     * that was requested will be fetched.
+     * @return void
+     */
+    void fetchChangedOnly( bool enable );
+
     /**
      * Returns the item fetch scope.
      *
@@ -235,8 +265,19 @@ class AKONADI_EXPORT Monitor : public QObject
      * Returns the set of items being monitored.
      *
      * @since 4.3
+     *
+     * @deprecated Use itemsMonitoredEx() instead.
      */
-    QList<Item::Id> itemsMonitored() const;
+    KDE_DEPRECATED QList<Item::Id> itemsMonitored() const;
+
+    /**
+     * Returns the set of items being monitored.
+     *
+     * Faster version (at least on 32-bit systems) of itemsMonitored().
+     *
+     * @since 4.6
+     */
+    QVector<Item::Id> itemsMonitoredEx() const;
 
     /**
      * Returns the set of mimetypes being monitored.
@@ -372,6 +413,27 @@ class AKONADI_EXPORT Monitor : public QObject
      */
     void collectionRemoved( const Akonadi::Collection &collection );
 
+    /**
+     * This signal is emitted if a collection has been subscribed to by the user.
+     *  It will be emitted even for unmonitored collections as the check for whether to
+     *  monitor it has not been applied yet.
+     *
+     * @param collection The subscribed collection
+     * @param parent The parent collection of the subscribed collection.
+     *
+     * @since 4.6
+     */
+    void collectionSubscribed( const Akonadi::Collection& collection, const Akonadi::Collection &parent  );
+
+    /**
+     * This signal is emitted if a user unsubscribes from a collection.
+     *
+     * @param collection The unsubscribed collection
+     *
+     * @since 4.6
+     */
+    void collectionUnsubscribed( const Akonadi::Collection& collection );
+
     /**
      * This signal is emitted if the statistics information of a monitored collection
      * has changed.
@@ -444,6 +506,9 @@ class AKONADI_EXPORT Monitor : public QObject
     Q_PRIVATE_SLOT( d_ptr, void slotFlushRecentlyChangedCollections() )
     Q_PRIVATE_SLOT( d_ptr, void slotNotify( const Akonadi::NotificationMessage::List& ) )
     Q_PRIVATE_SLOT( d_ptr, void dataAvailable() )
+    Q_PRIVATE_SLOT( d_ptr, void serverStateChanged( Akonadi::ServerManager::State ) )
+    Q_PRIVATE_SLOT( d_ptr, void invalidateCollectionCache( qint64 ) )
+    Q_PRIVATE_SLOT( d_ptr, void invalidateItemCache( qint64 ) )
 
     friend class ResourceBasePrivate;
     //@endcond
