@@ -19,6 +19,8 @@
 
 #include "resourcescheduler_p.h"
 
+#include "dbusconnectionpool.h"
+
 #include <kdebug.h>
 #include <klocale.h>
 
@@ -78,6 +80,20 @@ void ResourceScheduler::scheduleSync(const Collection & col)
   scheduleNext();
 }
 
+void ResourceScheduler::scheduleAttributesSync( const Collection &collection )
+{
+  Task t;
+  t.type = SyncCollectionAttributes;
+  t.collection = collection;
+
+  TaskList& queue = queueForTaskType( t.type );
+  if ( queue.contains( t ) || mCurrentTask == t )
+    return;
+  queue << t;
+  signalTaskToTracker( t, "SyncCollectionAttributes" );
+  scheduleNext();
+}
+
 void ResourceScheduler::scheduleItemFetch(const Item & item, const QSet<QByteArray> &parts, const QDBusMessage & msg)
 {
   Task t;
@@ -118,6 +134,19 @@ void ResourceScheduler::scheduleResourceCollectionDeletion()
   scheduleNext();
 }
 
+void ResourceScheduler::scheduleCacheInvalidation( const Collection &collection )
+{
+  Task t;
+  t.type = InvalideCacheForCollection;
+  t.collection = collection;
+  TaskList& queue = queueForTaskType( t.type );
+  if ( queue.contains( t ) || mCurrentTask == t )
+    return;
+  queue << t;
+  signalTaskToTracker( t, "InvalideCacheForCollection" );
+  scheduleNext();
+}
+
 void ResourceScheduler::scheduleChangeReplay()
 {
   Task t;
@@ -142,6 +171,17 @@ void Akonadi::ResourceScheduler::scheduleFullSyncCompletion()
   scheduleNext();
 }
 
+void Akonadi::ResourceScheduler::scheduleCollectionTreeSyncCompletion()
+{
+  Task t;
+  t.type = SyncCollectionTreeDone;
+  TaskList& queue = queueForTaskType( t.type );
+  // no compression here, all this does is emitting a D-Bus signal anyway, and compression can trigger races on the receiver side with the signal being lost
+  queue << t;
+  signalTaskToTracker( t, "SyncCollectionTreeDone" );
+  scheduleNext();
+}
+
 void Akonadi::ResourceScheduler::scheduleCustomTask( QObject *receiver, const char* methodName, const QVariant &argument, ResourceBase::SchedulePriority priority )
 {
   Task t;
@@ -226,7 +266,7 @@ void ResourceScheduler::scheduleNext()
 {
   if ( mCurrentTask.type != Invalid || isEmpty() || !mOnline )
     return;
-  QTimer::singleShot( 0, this, SLOT( executeNext() ) );
+  QTimer::singleShot( 0, this, SLOT(executeNext()) );
 }
 
 void ResourceScheduler::executeNext()
@@ -258,21 +298,36 @@ void ResourceScheduler::executeNext()
     case SyncCollection:
       emit executeCollectionSync( mCurrentTask.collection );
       break;
+    case SyncCollectionAttributes:
+      emit executeCollectionAttributesSync( mCurrentTask.collection );
+      break;
     case FetchItem:
       emit executeItemFetch( mCurrentTask.item, mCurrentTask.itemParts );
       break;
     case DeleteResourceCollection:
       emit executeResourceCollectionDeletion();
       break;
+    case InvalideCacheForCollection:
+      emit executeCacheInvalidation( mCurrentTask.collection );
+      break;
     case ChangeReplay:
       emit executeChangeReplay();
       break;
     case SyncAllDone:
       emit fullSyncComplete();
       break;
+    case SyncCollectionTreeDone:
+      emit collectionTreeSyncComplete();
+      break;
     case Custom:
     {
-      bool success = QMetaObject::invokeMethod( mCurrentTask.receiver, mCurrentTask.methodName, Q_ARG(QVariant, mCurrentTask.argument) );
+      const QByteArray methodSig = mCurrentTask.methodName + "(QVariant)";
+      const bool hasSlotWithVariant = mCurrentTask.receiver->metaObject()->indexOfMethod(methodSig) != -1;
+      bool success = false;
+      if ( hasSlotWithVariant ) {
+        success = QMetaObject::invokeMethod( mCurrentTask.receiver, mCurrentTask.methodName, Q_ARG(QVariant, mCurrentTask.argument) );
+        Q_ASSERT_X( success || !mCurrentTask.argument.isValid(), "ResourceScheduler::executeNext", "Valid argument was provided but the method wasn't found" );
+      }
       if ( !success )
         success = QMetaObject::invokeMethod( mCurrentTask.receiver, mCurrentTask.methodName );
 
@@ -329,11 +384,11 @@ void ResourceScheduler::setOnline(bool state)
 void ResourceScheduler::signalTaskToTracker( const Task &task, const QByteArray &taskType )
 {
   // if there's a job tracer running, tell it about the new job
-  if ( !s_resourcetracker && QDBusConnection::sessionBus().interface()->isServiceRegistered(QLatin1String( "org.kde.akonadiconsole" ) ) ) {
+  if ( !s_resourcetracker && DBusConnectionPool::threadConnection().interface()->isServiceRegistered(QLatin1String( "org.kde.akonadiconsole" ) ) ) {
     s_resourcetracker = new QDBusInterface( QLatin1String( "org.kde.akonadiconsole" ),
                                        QLatin1String( "/resourcesJobtracker" ),
                                        QLatin1String( "org.freedesktop.Akonadi.JobTracker" ),
-                                       QDBusConnection::sessionBus(), 0 );
+                                       DBusConnectionPool::threadConnection(), 0 );
   }
 
   if ( s_resourcetracker ) {
@@ -365,7 +420,7 @@ void ResourceScheduler::Task::sendDBusReplies( bool success )
   Q_FOREACH( const QDBusMessage &msg, dbusMsgs ) {
     QDBusMessage reply( msg );
     reply << success;
-    QDBusConnection::sessionBus().send( reply );
+    DBusConnectionPool::threadConnection().send( reply );
   }
 }
 
@@ -389,15 +444,27 @@ ResourceScheduler::TaskList& ResourceScheduler::queueForTaskType( TaskType type
 
 void ResourceScheduler::dump()
 {
-  kDebug() << "ResourceScheduler: Online:" << mOnline;
-  kDebug() << " current task:" << mCurrentTask;
+  kDebug() << dumpToString();
+}
+
+QString ResourceScheduler::dumpToString() const
+{
+  QString ret;
+  QTextStream str( &ret );
+  str << "ResourceScheduler: " << (mOnline?"Online":"Offline") << endl;
+  str << " current task: " << mCurrentTask << endl;
   for ( int i = 0; i < NQueueCount; ++i ) {
     const TaskList& queue = mTaskList[i];
-    kDebug() << " queue" << i << queue.size() << "tasks:";
-    for ( QList<Task>::const_iterator it = queue.begin(); it != queue.end(); ++it ) {
-      kDebug() << "  " << (*it);
+    if (queue.isEmpty()) {
+      str << " queue " << i << " is empty" << endl;
+    } else {
+      str << " queue " << i << " " << queue.size() << " tasks:" << endl;
+      for ( QList<Task>::const_iterator it = queue.begin(); it != queue.end(); ++it ) {
+        str << "  " << (*it) << endl;
+      }
     }
   }
+  return ret;
 }
 
 void ResourceScheduler::clear()
@@ -411,29 +478,56 @@ void ResourceScheduler::clear()
   mCurrentTasksQueue = -1;
 }
 
-static const char s_taskTypes[][25] = {
-      "Invalid",
+void Akonadi::ResourceScheduler::cancelQueues()
+{
+  for ( int i = 0; i < NQueueCount; ++i ) {
+    TaskList& queue = mTaskList[i];
+    if ( s_resourcetracker ) {
+      foreach ( const Task &t, queue ) {
+        QList<QVariant> argumentList;
+        argumentList << QString::number( t.serial ) << QString();
+        s_resourcetracker->asyncCallWithArgumentList(QLatin1String( "jobEnded" ), argumentList);
+      }
+    }
+    queue.clear();
+  }
+}
+
+static const char s_taskTypes[][27] = {
+      "Invalid (no task)",
       "SyncAll",
       "SyncCollectionTree",
       "SyncCollection",
+      "SyncCollectionAttributes",
       "FetchItem",
       "ChangeReplay",
       "DeleteResourceCollection",
+      "InvalideCacheForCollection",
       "SyncAllDone",
+      "SyncCollectionTreeDone",
       "Custom"
 };
 
+QTextStream& Akonadi::operator<<( QTextStream& d, const ResourceScheduler::Task& task )
+{
+  d << task.serial << " " << s_taskTypes[task.type] << " ";
+  if ( task.type != ResourceScheduler::Invalid ) {
+    if ( task.collection.isValid() )
+      d << "collection " << task.collection.id() << " ";
+    if ( task.item.id() != -1 )
+      d << "item " << task.item.id() << " ";
+    if ( !task.methodName.isEmpty() )
+      d << task.methodName << " " << task.argument.toString();
+  }
+  return d;
+}
+
 QDebug Akonadi::operator<<( QDebug d, const ResourceScheduler::Task& task )
 {
-  d << task.serial << s_taskTypes[task.type];
-  if ( task.type != ResourceScheduler::Invalid ) {
-    if ( task.collection.id() != -1 )
-      d << "collection" << task.collection.id();
-    if ( task.item.id() != -1 )
-      d << "item" << task.item.id();
-    if ( !task.methodName.isEmpty() )
-      d << task.methodName << task.argument;
-  }
+  QString s;
+  QTextStream str( &s );
+  str << task;
+  d << s;
   return d;
 }
 
