@@ -30,6 +30,7 @@
 #include <KDebug>
 
 Q_DECLARE_METATYPE(QList<KIMAP::MailBoxDescriptor>)
+Q_DECLARE_METATYPE(QList< QList<QByteArray> >)
 
 class ListJobTest: public QObject {
   Q_OBJECT
@@ -67,6 +68,31 @@ void testList_data() {
 
   QTest::newRow( "normal" ) << true << scenario << listresult;
 
+  scenario.clear();;
+  scenario << FakeServer::preauth()
+           << "C: A000001 LIST \"\" *"
+           << "S: * LIST ( \\HasChildren ) / Inbox"
+           << "S: * LIST ( \\HasNoChildren ) / Inbox/&AOQ- &APY- &APw- @ &IKw-"
+           << "S: * LIST ( \\HasChildren ) / Inbox/lost+found"
+           << "S: * LIST ( \\HasNoChildren ) / \"Inbox/lost+found/Calendar Public-20080128\""
+           << "S: A000001 OK LIST completed";
+  listresult.clear();
+
+  descriptor.separator = '/';
+  descriptor.name = "INBOX";
+  listresult << descriptor;
+  descriptor.separator = '/';
+  descriptor.name = QString::fromUtf8( "INBOX/ä ö ü @ €" );
+  listresult << descriptor;
+  descriptor.separator = '/';
+  descriptor.name = "INBOX/lost+found";
+  listresult << descriptor;
+  descriptor.separator = '/';
+  descriptor.name = "INBOX/lost+found/Calendar Public-20080128";
+  listresult << descriptor;
+
+  QTest::newRow( "lowercase Inbox" ) << true << scenario << listresult;
+
   scenario.clear();
   scenario << FakeServer::preauth()
            << "C: A000001 LSUB \"\" *"
@@ -96,6 +122,35 @@ void testList_data() {
 
   QTest::newRow( "subscribed" ) << false << scenario << listresult;
 
+ scenario.clear();
+  scenario << FakeServer::preauth()
+           << "C: A000001 LSUB \"\" *"
+           << "S: * LSUB ( \\HasChildren ) / Inbox"
+           << "S: * LSUB ( ) / Inbox/Calendar/3196"
+           << "S: * LSUB ( \\HasChildren ) / Inbox/Calendar/ff"
+           << "S: * LSUB ( ) / Inbox/Calendar/ff/hgh"
+           << "S: * LSUB ( ) / user/test2/Calendar"
+           << "S: A000001 OK LSUB completed";
+  listresult.clear();
+
+  descriptor.separator = '/';
+  descriptor.name = "INBOX";
+  listresult << descriptor;
+  descriptor.separator = '/';
+  descriptor.name = "INBOX/Calendar/3196";
+  listresult << descriptor;
+  descriptor.separator = '/';
+  descriptor.name = "INBOX/Calendar/ff";
+  listresult << descriptor;
+  descriptor.separator = '/';
+  descriptor.name = "INBOX/Calendar/ff/hgh";
+  listresult << descriptor;
+  descriptor.separator = '/';
+  descriptor.name = "user/test2/Calendar";
+  listresult << descriptor;
+
+  QTest::newRow( "subscribed, lowercase Inbox" ) << false << scenario << listresult;
+
   scenario.clear();
   scenario << FakeServer::preauth()
            << "C: A000001 LIST \"\" *"
@@ -108,6 +163,18 @@ void testList_data() {
 
   QTest::newRow( "unquoted-space" ) << true << scenario << listresult;
 
+  scenario.clear();
+  scenario << FakeServer::preauth()
+           << "C: A000001 LIST \"\" *"
+           << "S: * LIST ( \\NoInferiors ) ( ) INBOX"
+           << "S: A000001 OK LIST completed";
+  listresult.clear();
+  descriptor.separator = '/';
+  descriptor.name = "INBOX";
+  listresult << descriptor;
+
+  QTest::newRow( "separator is empty list" ) << true << scenario << listresult;
+
   scenario.clear();
   scenario << FakeServer::preauth()
            << "C: A000001 LIST \"\" *"
@@ -136,15 +203,26 @@ void testList()
 
     KIMAP::ListJob *job = new KIMAP::ListJob(&session);
     job->setIncludeUnsubscribed(unsubscribed);
+
+    QSignalSpy spy( job,
+                    SIGNAL(mailBoxesReceived(const QList<KIMAP::MailBoxDescriptor>&,
+                                             const QList< QList<QByteArray> >&)) );
+
     bool result = job->exec();
     QEXPECT_FAIL("bad" , "Expected failure on BAD response", Continue);
     QEXPECT_FAIL("no" , "Expected failure on NO response", Continue);
     QVERIFY(result);
     if (result) {
-      //kDebug() << job->mailBoxes().first().name;
+      QVERIFY( spy.count()>0 );
+      QList<KIMAP::MailBoxDescriptor> mailBoxes;
+
+      for ( int i=0; i<spy.count(); i++ ) {
+        mailBoxes+= spy.at( i ).at( 0 ).value< QList<KIMAP::MailBoxDescriptor> >();
+      }
+
+      //kDebug() << mailBoxes.first().name;
       //kDebug() << listresult.first().name;
-      QCOMPARE(job->mailBoxes(), listresult);
-      //       kDebug() << job->flags();
+      QCOMPARE(mailBoxes, listresult);
     }
 //     QCOMPARE(job->mailBox(), mailbox);
 
