@@ -166,6 +166,16 @@ void HeaderTest::testAddressListHeader()
   QCOMPARE( h->asUnicodeString(), testAddress );
   delete h;
 
+  {
+    // a display name with non-latin1 content in both name components
+    h = new Headers::Generics::AddressList();
+    const QString testAddress = QString::fromUtf8( "\"Rüedi-Huser, Thomas\" <test@test.org>" );
+    h->fromUnicodeString( testAddress, "utf-8" );
+    QEXPECT_FAIL( "", "AddressList::prettyAddresses() does not quote the mailbox correctly", Continue );
+    QCOMPARE( h->asUnicodeString(), testAddress );
+    delete h;
+  }
+
   // again, this time legacy style
   h = new Headers::Generics::AddressList();
   h->from7BitString( "kloecker@kde.org (Ingo =?iso-8859-15?q?Kl=F6cker?=)" );
@@ -582,6 +592,21 @@ void HeaderTest::testContentTypeHeader()
   h = new ContentType( 0, "text/plain;\n name==?ISO-8859-1?Q?lor=E9m_ipsum=2Etxt?=" );
   QCOMPARE( h->name(), QString::fromUtf8( "lorém ipsum.txt" ) );
   delete h;
+
+  // make ervin's unit test happy
+  h = new ContentType;
+  h->setMimeType( "MULTIPART/MIXED" );
+  QVERIFY( h->isMultipart() );
+  QVERIFY( h->isMediatype( "multipart" ) );
+  QVERIFY( h->isMediatype( "Multipart" ) );
+  QVERIFY( h->isMediatype( "MULTIPART" ) );
+  QVERIFY( h->isSubtype( "mixed" ) );
+  QVERIFY( h->isSubtype( "Mixed" ) );
+  QVERIFY( h->isSubtype( "MIXED" ) );
+  QCOMPARE( h->mimeType(), QByteArray( "MULTIPART/MIXED" ) );
+  QCOMPARE( h->mediaType(), QByteArray( "MULTIPART" ) );
+  QCOMPARE( h->subType(), QByteArray( "MIXED" ) );
+  delete h;
 }
 
 void HeaderTest::testTokenHeader()
@@ -899,7 +924,8 @@ void HeaderTest::testInvalidQEncoding()
   using namespace HeaderParsing;
   QFETCH( QString,encodedWord );
 
-  const char *data = encodedWord.toAscii().data();
+  QByteArray tmp = encodedWord.toAscii();
+  const char *data = tmp.data();
   const char *start = data + 1;
   const char *end = data + strlen( data );
   QString result;
@@ -908,4 +934,53 @@ void HeaderTest::testInvalidQEncoding()
   QVERIFY( !parseEncodedWord( start, end, result, language, usedCS ) );
 }
 
+void HeaderTest::testBug271192()
+{
+  QFETCH( QString, displayName );
+  QFETCH( bool, quote );
+
+  const QString addrSpec = QLatin1String( "example@example.com" );
+  const QString mailbox = (quote ? QLatin1String("\"") : QString()) + displayName +
+                          (quote ? QLatin1String("\"") : QString()) +
+                          QLatin1String(" <") + addrSpec + QLatin1String(">");
+
+  Headers::Generics::SingleMailbox *h = new Headers::Generics::SingleMailbox();
+  h->fromUnicodeString( mailbox, "utf-8" );
+  QCOMPARE( h->displayNames().size(), 1 );
+  QCOMPARE( h->displayNames().first().toUtf8(), displayName.remove( QLatin1String("\\") ).toUtf8() );
+  delete h;
+  h = 0;
+
+  Headers::Generics::MailboxList *h2 = new Headers::Generics::MailboxList();
+  h2->fromUnicodeString( mailbox + QLatin1String(",") + mailbox, "utf-8" );
+  QCOMPARE( h2->displayNames().size(), 2 );
+  QCOMPARE( h2->displayNames()[0].toUtf8(), displayName.remove( QLatin1String("\\") ).toUtf8() );
+  QCOMPARE( h2->displayNames()[1].toUtf8(), displayName.remove( QLatin1String("\\") ).toUtf8() );
+  delete h2;
+  h2 = 0;
+}
+
+void HeaderTest::testBug271192_data()
+{
+  QTest::addColumn<QString>("displayName");
+  QTest::addColumn<bool>("quote");
+
+  QTest::newRow("Plain") << QString::fromUtf8( "John Doe" ) << false;
+  QTest::newRow("Firstname 1") << QString::fromUtf8( "Marc-André Lastname" ) << false;
+  QTest::newRow("Firstname 2") << QString::fromUtf8( "Интернет-компания Lastname" ) << false;
+  QTest::newRow("Lastname") << QString::fromUtf8( "Tobias König" ) << false;
+  QTest::newRow("Firstname + Lastname") << QString::fromUtf8( "Интернет-компания König" ) << false;
+  QTest::newRow("Quotemarks") << QString::fromUtf8( "John \\\"Rocky\\\" Doe" ) << true;
+  QTest::newRow("Quotemarks") << QString::fromUtf8( "Jöhn \\\"Röcky\\\" Döe" ) << true;
+
+  QTest::newRow("Plain") << QString::fromUtf8( "John Doe" ) << true;
+  QTest::newRow("Firstname 1") << QString::fromUtf8( "Marc-André Lastname" ) << true;
+  QTest::newRow("Firstname 2") << QString::fromUtf8( "Интернет-компания Lastname" ) << true;
+  QTest::newRow("Lastname") << QString::fromUtf8( "Tobias König" ) << true;
+  QTest::newRow("Firstname + Lastname") << QString::fromUtf8( "Интернет-компания König" ) << true;
+  QTest::newRow("LastName, Firstname") << QString::fromUtf8( "König, Интернет-компания" ) << true;
+}
+
+
+
 #include "headertest.moc"
